'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _webpack = require('webpack');

var _webpack2 = _interopRequireDefault(_webpack);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _logUpdate = require('log-update');

var _logUpdate2 = _interopRequireDefault(_logUpdate);

var _profile = require('./profile');

var _profile2 = _interopRequireDefault(_profile);

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const sharedState = {};

const defaults = {
  name: 'webpack',
  color: 'green',
  stream: process.stdout,
  profile: false,
  clear: false,
  showCursor: false
};

class WebpackBarPlugin extends _webpack2.default.ProgressPlugin {
  constructor(options) {
    super();

    this.options = Object.assign({}, defaults, options);

    this.handler = (percent, msg, ...details) => this.updateProgress(percent, msg, details);

    // Don't throttle when profiling
    if (!this.options.profile) {
      this.handler = _lodash2.default.throttle(this.handler, 25, { leading: true, trailing: true });
    }

    this.logUpdate = this.options.logUpdate || _logUpdate2.default.create(this.options.stream, {
      showCursor: this.options.showCursor
    });

    if (!sharedState[this.options.name]) {
      sharedState[this.options.name] = {
        color: this.options.color,
        profile: this.options.profile ? new _profile2.default(this.options.name) : null
      };
    }
  }

  apply(compiler) {
    super.apply(compiler);

    if (compiler.hooks) {
      // Webpack >= 4
      compiler.hooks.done.tap('webpackbar', () => this.done());
    } else {
      // Webpack < 4
      compiler.plugin('done', () => this.done());
    }
  }

  done() {
    if (this.options.clear) {
      _logUpdate2.default.clear();
    }

    if (this.options.profile) {
      const stats = sharedState[this.options.name].profile.getStats();
      (0, _utils.printStats)(stats);
    }
  }

  updateProgress(percent, msg, details) {
    const progress = Math.floor(percent * 100);

    Object.assign(sharedState[this.options.name], {
      progress,
      msg,
      details: details || [],
      request: (0, _utils.parseRequst)(details[2]),
      isRunning: progress && progress !== 100 && msg && msg.length
    });

    if (this.options.profile) {
      sharedState[this.options.name].profile.onRequest(sharedState[this.options.name].request);
    }

    // Process all states
    let isRunning = false;

    const lines = [];

    _lodash2.default.sortBy(Object.keys(sharedState), s => s.name).reverse().forEach(name => {
      const state = sharedState[name];

      if (state.isRunning) {
        isRunning = true;
      } else if (this.options.clear) {
        // Hide finished jobs
        return;
      }

      const lColor = (0, _utils.colorize)(state.color);
      const lIcon = lColor(_utils.BULLET);
      const lName = lColor(_lodash2.default.startCase(name));
      const lBar = (0, _utils.renderBar)(state.progress, state.color);
      const lMsg = _lodash2.default.startCase(state.msg);
      const lProgress = `(${state.progress || 0}%)`;
      const lDetail1 = _chalk2.default.grey(state.details && state.details[0] || '');
      const lDetail2 = _chalk2.default.grey(state.details && state.details[1] || '');
      const lRequest = state.request ? (0, _utils.formatRequest)(state.request) : '';

      lines.push(`${[lIcon, lName, lBar, lMsg, lProgress, lDetail1, lDetail2].join(' ')}\n ${lRequest}`);
    });

    if (!isRunning) {
      if (this.options.clear) {
        this.logUpdate.clear();
      }
    } else {
      const title = ` ${_chalk2.default.bgBlue.black(' BUILDING ')}`;
      this.logUpdate(`\n${title}\n\n${lines.join('\n\n')}`);
    }
  }
}
exports.default = WebpackBarPlugin;